package java.awt;

import sun.font.FontAccess;
import sun.font.GlyphLayout;
import java.awt.font.GlyphVector;
import sun.font.StandardGlyphVector;
import java.awt.font.TextLayout;
import sun.font.FontUtilities;
import java.awt.geom.Rectangle2D;
import java.awt.font.LineMetrics;
import sun.font.CoreMetrics;
import java.awt.geom.Point2D;
import java.awt.font.FontRenderContext;
import java.text.CharacterIterator;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.FilePermission;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.security.AccessController;
import java.security.PrivilegedExceptionAction;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.attribute.FileAttribute;
import sun.font.EAttribute;
import java.awt.font.TextAttribute;
import sun.font.AttributeMap;
import java.text.AttributedCharacterIterator;
import java.util.Map;
import java.util.Locale;
import sun.font.CreatedFontTracker;
import java.io.File;
import sun.font.FontManager;
import sun.font.CompositeFont;
import sun.font.FontManagerFactory;
import sun.font.Font2D;
import sun.font.FontLineMetrics;
import java.lang.ref.SoftReference;
import java.awt.geom.AffineTransform;
import sun.font.AttributeValues;
import sun.font.Font2DHandle;
import java.awt.peer.FontPeer;
import java.util.Hashtable;
import java.io.Serializable;

public class Font implements Serializable
{
    private Hashtable<Object, Object> fRequestedAttributes;
    public static final String DIALOG = "Dialog";
    public static final String DIALOG_INPUT = "DialogInput";
    public static final String SANS_SERIF = "SansSerif";
    public static final String SERIF = "Serif";
    public static final String MONOSPACED = "Monospaced";
    public static final int PLAIN = 0;
    public static final int BOLD = 1;
    public static final int ITALIC = 2;
    public static final int ROMAN_BASELINE = 0;
    public static final int CENTER_BASELINE = 1;
    public static final int HANGING_BASELINE = 2;
    public static final int TRUETYPE_FONT = 0;
    public static final int TYPE1_FONT = 1;
    protected String name;
    protected int style;
    protected int size;
    protected float pointSize;
    private transient FontPeer peer;
    private transient long pData;
    private transient Font2DHandle font2DHandle;
    private transient AttributeValues values;
    private transient boolean hasLayoutAttributes;
    private transient boolean createdFont;
    private transient boolean nonIdentityTx;
    private static final AffineTransform identityTx;
    private static final long serialVersionUID = -4206021311591459213L;
    private static final int RECOGNIZED_MASK;
    private static final int PRIMARY_MASK;
    private static final int SECONDARY_MASK;
    private static final int LAYOUT_MASK;
    private static final int EXTRA_MASK;
    private static final float[] ssinfo;
    transient int hash;
    private int fontSerializedDataVersion;
    private transient SoftReference<FontLineMetrics> flmref;
    public static final int LAYOUT_LEFT_TO_RIGHT = 0;
    public static final int LAYOUT_RIGHT_TO_LEFT = 1;
    public static final int LAYOUT_NO_START_CONTEXT = 2;
    public static final int LAYOUT_NO_LIMIT_CONTEXT = 4;
    
    @Deprecated
    public FontPeer getPeer() {
        return this.getPeer_NoClientCode();
    }
    
    final FontPeer getPeer_NoClientCode() {
        if (this.peer == null) {
            this.peer = Toolkit.getDefaultToolkit().getFontPeer(this.name, this.style);
        }
        return this.peer;
    }
    
    private AttributeValues getAttributeValues() {
        if (this.values == null) {
            AttributeValues values = new AttributeValues();
            values.setFamily(this.name);
            values.setSize(this.pointSize);
            if ((this.style & 0x1) != 0x0) {
                values.setWeight(2.0f);
            }
            if ((this.style & 0x2) != 0x0) {
                values.setPosture(0.2f);
            }
            values.defineAll(Font.PRIMARY_MASK);
            this.values = values;
        }
        return this.values;
    }
    
    private Font2D getFont2D() {
        FontManager instance = FontManagerFactory.getInstance();
        if (instance.usingPerAppContextComposites() && this.font2DHandle != null && this.font2DHandle.font2D instanceof CompositeFont && ((CompositeFont)(CompositeFont)this.font2DHandle.font2D).isStdComposite()) {
            return instance.findFont2D(this.name, this.style, 2);
        }
        if (this.font2DHandle == null) {
            this.font2DHandle = instance.findFont2D(this.name, this.style, 2).handle;
        }
        return this.font2DHandle.font2D;
    }
    
    public Font(String s, int n, int size) {
        this.createdFont = false;
        this.fontSerializedDataVersion = 1;
        this.name = ((s != null) ? s : "Default");
        this.style = (((n & 0xFFFFFFFC) == 0x0) ? n : 0);
        this.size = size;
        this.pointSize = (float)size;
    }
    
    private Font(String s, int n, float pointSize) {
        this.createdFont = false;
        this.fontSerializedDataVersion = 1;
        this.name = ((s != null) ? s : "Default");
        this.style = (((n & 0xFFFFFFFC) == 0x0) ? n : 0);
        this.size = (int)((double)pointSize + 0.5);
        this.pointSize = pointSize;
    }
    
    private Font(String s, int n, float n2, boolean createdFont, Font2DHandle font2DHandle) {
        this(s, n, n2);
        this.createdFont = createdFont;
        if (createdFont) {
            if (font2DHandle.font2D instanceof CompositeFont && font2DHandle.font2D.getStyle() != n) {
                this.font2DHandle = FontManagerFactory.getInstance().getNewComposite(null, n, font2DHandle);
            }
            else {
                this.font2DHandle = font2DHandle;
            }
        }
    }
    
    private Font(File file, int n, boolean b, CreatedFontTracker createdFontTracker) throws FontFormatException {
        this.createdFont = false;
        this.fontSerializedDataVersion = 1;
        this.createdFont = true;
        this.font2DHandle = FontManagerFactory.getInstance().createFont2D(file, n, b, createdFontTracker).handle;
        this.name = this.font2DHandle.font2D.getFontName(Locale.getDefault());
        this.style = 0;
        this.size = 1;
        this.pointSize = 1.0f;
    }
    
    private Font(AttributeValues attributeValues, String s, int n, boolean createdFont, Font2DHandle font2DHandle) {
        this.createdFont = false;
        this.fontSerializedDataVersion = 1;
        this.createdFont = createdFont;
        if (createdFont) {
            this.font2DHandle = font2DHandle;
            String family = null;
            if (s != null) {
                family = attributeValues.getFamily();
                if (s.equals(family)) {
                    family = null;
                }
            }
            int n2 = 0;
            if (n == -1) {
                n2 = -1;
            }
            else {
                if (attributeValues.getWeight() >= 2.0f) {
                    n2 = 1;
                }
                if (attributeValues.getPosture() >= 0.2f) {
                    n2 |= 0x2;
                }
                if (n == n2) {
                    n2 = -1;
                }
            }
            if ((font2DHandle.font2D instanceof CompositeFont)) {
                if (n2 != -1 || family != null) {
                    this.font2DHandle = FontManagerFactory.getInstance().getNewComposite(family, n2, font2DHandle);
                }
            }
            else if (family != null) {
                this.createdFont = false;
                this.font2DHandle = null;
            }
        }
        this.initFromValues(attributeValues);
    }
    
    public Font(Map<? extends AttributedCharacterIterator.Attribute, ?> map) {
        this.createdFont = false;
        this.fontSerializedDataVersion = 1;
        this.initFromValues(AttributeValues.fromMap(map, Font.RECOGNIZED_MASK));
    }
    
    protected Font(Font font) {
        this.createdFont = false;
        this.fontSerializedDataVersion = 1;
        if (font.values != null) {
            this.initFromValues(font.getAttributeValues().clone());
        }
        else {
            this.name = font.name;
            this.style = font.style;
            this.size = font.size;
            this.pointSize = font.pointSize;
        }
        this.font2DHandle = font.font2DHandle;
        this.createdFont = font.createdFont;
    }
    
    private void initFromValues(AttributeValues values) {
        (this.values = values).defineAll(Font.PRIMARY_MASK);
        this.name = values.getFamily();
        this.pointSize = values.getSize();
        this.size = (int)((double)values.getSize() + 0.5);
        if (values.getWeight() >= 2.0f) {
            this.style |= 0x1;
        }
        if (values.getPosture() >= 0.2f) {
            this.style |= 0x2;
        }
        this.nonIdentityTx = values.anyNonDefault(Font.EXTRA_MASK);
        this.hasLayoutAttributes = values.anyNonDefault(Font.LAYOUT_MASK);
    }
    
    public static Font getFont(Map<? extends AttributedCharacterIterator.Attribute, ?> map) {
        if (map instanceof AttributeMap && ((AttributeMap)map).getValues() != null) {
            AttributeValues values = ((AttributeMap)map).getValues();
            if (values.isNonDefault(EAttribute.EFONT)) {
                Font font2 = values.getFont();
                if (!values.anyDefined(Font.SECONDARY_MASK)) {
                    return font2;
                }
                AttributeValues clone2 = font2.getAttributeValues().clone();
                clone2.merge(map, Font.SECONDARY_MASK);
                return new Font(clone2, font2.name, font2.style, font2.createdFont, font2.font2DHandle);
            }
            else {
                return new Font(map);
            }
        }
        else {
            Font font = (Font)map.get(TextAttribute.FONT);
            if (font != null) {
                if (map.size() > 1) {
                    AttributeValues clone = font.getAttributeValues().clone();
                    clone.merge(map, Font.SECONDARY_MASK);
                    return new Font(clone, font.name, font.style, font.createdFont, font.font2DHandle);
                }
                return font;
            }
            else {
                return new Font(map);
            }
        }
    }
    
    private static boolean hasTempPermission() {
        if (System.getSecurityManager() == null) {
            return true;
        }
        boolean n = false;
        try {
            Files.createTempFile("+~JT", ".tmp", new FileAttribute[0]).toFile().delete();
            n = true;
        }
        catch (Throwable t) {}
        return n;
    }
    
    public static Font createFont(int n, InputStream inputStream) throws FontFormatException, IOException {
        if (hasTempPermission()) {
            return createFont0(n, inputStream, null);
        }
        CreatedFontTracker tracker = CreatedFontTracker.getTracker();
        boolean acquirePermit = false;
        try {
            acquirePermit = tracker.acquirePermit();
            if (acquirePermit == 0) {
                throw new IOException("Timed out waiting for resources.");
            }
            Font font0 = createFont0(n, inputStream, tracker);
        }
        catch (InterruptedException ex) {
            throw new IOException("Problem reading font data.");
        }
        finally {
            if (!(acquirePermit == 0)) {
                tracker.releasePermit();
            }
        }
    }
    
    private static Font createFont0(int n, InputStream inputStream, CreatedFontTracker createdFontTracker) throws FontFormatException, IOException {
        if (n != 0 && n != 1) {
            throw new IllegalArgumentException("font format not recognized");
        }
        boolean n2 = false;
        try {
            final File file = (File)AccessController.doPrivileged(new PrivilegedExceptionAction<File>() {
                @Override
                public File run() throws IOException {
                    return Files.createTempFile("+~JF", ".tmp", new FileAttribute[0]).toFile();
                }
            });
            if (createdFontTracker != null) {
                createdFontTracker.add(file);
            }
            int sz = 0;
            try {
                OutputStream os = (OutputStream)AccessController.doPrivileged(new PrivilegedExceptionAction<OutputStream>() {
                    @Override
                    public OutputStream run() throws IOException {
                        return new FileOutputStream(file);
                    }
                });
                if (createdFontTracker != null) {
                    createdFontTracker.set(file, os);
                }
                try {
                    byte[] array = new byte[8192];
                    while (true) {
                        int read = inputStream.read(array);
                        if (read < 0) {
                            break;
                        }
                        if (createdFontTracker != null) {
                            if (sz + read > 33554432) {
                                throw new IOException("File too big.");
                            }
                            if (sz + createdFontTracker.getNumBytes() > 335544320) {
                                throw new IOException("Total files too big.");
                            }
                            sz += read;
                            createdFontTracker.addBytes(read);
                        }
                        os.write(array, 0, read);
                    }
                }
                finally {
                    os.close();
                }
                n2 = true;
                Font font = new Font(file, n, true, createdFontTracker);
            }
            finally {
                if (createdFontTracker != null) {
                    createdFontTracker.remove(file);
                }
                if (n2 == 0) {
                    if (createdFontTracker != null) {
                        createdFontTracker.subBytes(sz);
                    }
                    AccessController.doPrivileged(new PrivilegedExceptionAction<Void>() {
                        @Override
                        public Void run() {
                            file.delete();
                            return null;
                        }
                    });
                }
            }
        }
        catch (Throwable t) {
            if ((t instanceof FontFormatException)) {
                throw (FontFormatException)t;
            }
            if ((t instanceof IOException)) {
                throw (IOException)t;
            }
            Throwable cause = t.getCause();
            if ((cause instanceof FontFormatException)) {
                throw (FontFormatException)cause;
            }
            throw new IOException("Problem reading font data.");
        }
    }
    
    public static Font createFont(int n, File obj) throws FontFormatException, IOException {
        obj = new File(obj.getPath());
        if (n != 0 && n != 1) {
            throw new IllegalArgumentException("font format not recognized");
        }
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new FilePermission(obj.getPath(), "read"));
        }
        if (!obj.canRead()) {
            throw new IOException("Can't read " + obj);
        }
        return new Font(obj, n, false, null);
    }
    
    public AffineTransform getTransform() {
        if (this.nonIdentityTx) {
            AttributeValues attributeValues = this.getAttributeValues();
            AffineTransform affineTransform = attributeValues.isNonDefault(EAttribute.ETRANSFORM) ? new AffineTransform(attributeValues.getTransform()) : new AffineTransform();
            if (attributeValues.getSuperscript() != 0) {
                int superscript = attributeValues.getSuperscript();
                double n = 0.0;
                int n2 = 0;
                int n3 = superscript > 0;
                int n6;
                for (int n4 = n3 ? -1 : 1, n5 = n3 ? superscript : (-superscript); (n5 & 0x7) > n2; n5 >>= 3, n4 = -n4, n2 = n6) {
                    n6 = (n5 & 0x7);
                    n += (double)((float)n4 * (Font.ssinfo[n6] - Font.ssinfo[n2]));
                }
                double ty = n * (double)this.pointSize;
                double pow = Math.pow(0.6666666666666666, (double)n2);
                affineTransform.preConcatenate(AffineTransform.getTranslateInstance(0.0, ty));
                affineTransform.scale(pow, pow);
            }
            if (attributeValues.isNonDefault(EAttribute.EWIDTH)) {
                affineTransform.scale((double)attributeValues.getWidth(), 1.0);
                return affineTransform;
            }
            return affineTransform;
        }
        else {
            return new AffineTransform();
        }
    }
    
    public String getFamily() {
        return this.getFamily_NoClientCode();
    }
    
    final String getFamily_NoClientCode() {
        return this.getFamily(Locale.getDefault());
    }
    
    public String getFamily(Locale l) {
        if (l == null) {
            throw new NullPointerException("null locale doesn't mean default");
        }
        return this.getFont2D().getFamilyName(l);
    }
    
    public String getPSName() {
        return this.getFont2D().getPostscriptName();
    }
    
    public String getName() {
        return this.name;
    }
    
    public String getFontName() {
        return this.getFontName(Locale.getDefault());
    }
    
    public String getFontName(Locale l) {
        if (l == null) {
            throw new NullPointerException("null locale doesn't mean default");
        }
        return this.getFont2D().getFontName(l);
    }
    
    public int getStyle() {
        return this.style;
    }
    
    public int getSize() {
        return this.size;
    }
    
    public float getSize2D() {
        return this.pointSize;
    }
    
    public boolean isPlain() {
        return this.style == 0;
    }
    
    public boolean isBold() {
        return (this.style & 0x1) != 0x0;
    }
    
    public boolean isItalic() {
        return (this.style & 0x2) != 0x0;
    }
    
    public boolean isTransformed() {
        return this.nonIdentityTx;
    }
    
    public boolean hasLayoutAttributes() {
        return this.hasLayoutAttributes;
    }
    
    public static Font getFont(String s) {
        return getFont(s, null);
    }
    
    public static Font decode(String s) {
        int intValue = 12;
        int n = 0;
        if (s == null) {
            return new Font("Dialog", n, intValue);
        }
        int n2 = (s.lastIndexOf(45) > s.lastIndexOf(32)) ? '-' : ' ';
        int n3 = s.lastIndexOf(n2);
        n3 = s.lastIndexOf(n2, n3 - 1);
        int length = s.length();
        if (n3 > 0 && n3 + 1 < length) {
            try {
                intValue = (int)Integer.valueOf(s.substring(n3 + 1));
                if (intValue <= 0) {
                    intValue = 12;
                }
            }
            catch (NumberFormatException ex) {
                n3 = length;
                if (s.charAt(n3 - 1) == n2) {
                    --n3;
                }
            }
        }
        String s2;
        if (n3 >= 0 && n3 + 1 < length) {
            String lowerCase = s.substring(n3 + 1, n3).toLowerCase(Locale.ENGLISH);
            if (lowerCase.equals("bolditalic")) {
                n = 3;
            }
            else if (lowerCase.equals("italic")) {
                n = 2;
            }
            else if (lowerCase.equals("bold")) {
                n = 1;
            }
            else if (lowerCase.equals("plain")) {
                n = 0;
            }
            else {
                if (s.charAt(n3 - 1) == n2) {
                    --n3;
                }
            }
            s2 = s.substring(0, n3);
        }
        else {
            if (n3 > 0) {
                length = n3;
            }
            else if (n3 > 0) {
                length = n3;
            }
            if (length > 0 && s.charAt(length - 1) == n2) {
                --length;
            }
            s2 = s.substring(0, length);
        }
        return new Font(s2, n, intValue);
    }
    
    public static Font getFont(String key, Font font) {
        String property = null;
        try {
            property = System.getProperty(key);
        }
        catch (SecurityException ex) {}
        if (property == null) {
            return font;
        }
        return decode(property);
    }
    
    @Override
    public int hashCode() {
        if (this.hash == 0) {
            this.hash = (this.name.hashCode() ^ this.style ^ this.size);
            if (this.nonIdentityTx && this.values != null && this.values.getTransform() != null) {
                this.hash ^= this.values.getTransform().hashCode();
            }
        }
        return this.hash;
    }
    
    @Override
    public boolean equals(Object o) {
        if (o == this) {
            return true;
        }
        if (o != null) {
            try {
                Font font = (Font)o;
                if (this.size == font.size && this.style == font.style && this.nonIdentityTx == font.nonIdentityTx && this.hasLayoutAttributes == font.hasLayoutAttributes && this.pointSize == font.pointSize && this.name.equals(font.name)) {
                    if (this.values == null) {
                        return font.values == null || this.getAttributeValues().equals(font.values);
                    }
                    return this.values.equals(font.getAttributeValues());
                }
            }
            catch (ClassCastException ex) {}
        }
        return false;
    }
    
    @Override
    public String toString() {
        String str;
        if (this.isBold()) {
            str = (this.isItalic() ? "bolditalic" : "bold");
        }
        else {
            str = (this.isItalic() ? "italic" : "plain");
        }
        return this.getClass().getName() + "[family=" + this.getFamily() + ",name=" + this.name + ",style=" + str + ",size=" + this.size + "]";
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws ClassNotFoundException, IOException {
        if (this.values != null) {
            synchronized (this.values) {
                this.fRequestedAttributes = this.values.toSerializableHashtable();
                objectOutputStream.defaultWriteObject();
                this.fRequestedAttributes = null;
            }
        }
        else {
            objectOutputStream.defaultWriteObject();
        }
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws ClassNotFoundException, IOException {
        objectInputStream.defaultReadObject();
        if (this.pointSize == 0.0f) {
            this.pointSize = (float)this.size;
        }
        if (this.fRequestedAttributes != null) {
            try {
                this.values = this.getAttributeValues();
                AttributeValues fromSerializableHashtable = AttributeValues.fromSerializableHashtable(this.fRequestedAttributes);
                if (!AttributeValues.is16Hashtable(this.fRequestedAttributes)) {
                    fromSerializableHashtable.unsetDefault();
                }
                this.values = this.getAttributeValues().merge(fromSerializableHashtable);
                this.nonIdentityTx = this.values.anyNonDefault(Font.EXTRA_MASK);
                this.hasLayoutAttributes = this.values.anyNonDefault(Font.LAYOUT_MASK);
            }
            catch (Throwable cause) {
                throw new IOException(cause);
            }
            finally {
                this.fRequestedAttributes = null;
            }
        }
    }
    
    public int getNumGlyphs() {
        return this.getFont2D().getNumGlyphs();
    }
    
    public int getMissingGlyphCode() {
        return this.getFont2D().getMissingGlyphCode();
    }
    
    public byte getBaselineFor(char c) {
        return this.getFont2D().getBaselineFor(c);
    }
    
    public Map<TextAttribute, ?> getAttributes() {
        return new AttributeMap(this.getAttributeValues());
    }
    
    public AttributedCharacterIterator.Attribute[] getAvailableAttributes() {
        return new AttributedCharacterIterator.Attribute[] { TextAttribute.FAMILY, TextAttribute.WEIGHT, TextAttribute.WIDTH, TextAttribute.POSTURE, TextAttribute.SIZE, TextAttribute.TRANSFORM, TextAttribute.SUPERSCRIPT, TextAttribute.CHAR_REPLACEMENT, TextAttribute.FOREGROUND, TextAttribute.BACKGROUND, TextAttribute.UNDERLINE, TextAttribute.STRIKETHROUGH, TextAttribute.RUN_DIRECTION, TextAttribute.BIDI_EMBEDDING, TextAttribute.JUSTIFICATION, TextAttribute.INPUT_METHOD_HIGHLIGHT, TextAttribute.INPUT_METHOD_UNDERLINE, TextAttribute.SWAP_COLORS, TextAttribute.NUMERIC_SHAPING, TextAttribute.KERNING, TextAttribute.LIGATURES, TextAttribute.TRACKING };
    }
    
    public Font deriveFont(int n, float size) {
        if (this.values == null) {
            return new Font(this.name, n, size, this.createdFont, this.font2DHandle);
        }
        AttributeValues clone = this.getAttributeValues().clone();
        int n2 = (this.style != n) ? this.style : -1;
        applyStyle(n, clone);
        clone.setSize(size);
        return new Font(clone, null, n2, this.createdFont, this.font2DHandle);
    }
    
    public Font deriveFont(int n, AffineTransform affineTransform) {
        AttributeValues clone = this.getAttributeValues().clone();
        int n2 = (this.style != n) ? this.style : -1;
        applyStyle(n, clone);
        applyTransform(affineTransform, clone);
        return new Font(clone, null, n2, this.createdFont, this.font2DHandle);
    }
    
    public Font deriveFont(float size) {
        if (this.values == null) {
            return new Font(this.name, this.style, size, this.createdFont, this.font2DHandle);
        }
        AttributeValues clone = this.getAttributeValues().clone();
        clone.setSize(size);
        return new Font(clone, null, -1, this.createdFont, this.font2DHandle);
    }
    
    public Font deriveFont(AffineTransform affineTransform) {
        AttributeValues clone = this.getAttributeValues().clone();
        applyTransform(affineTransform, clone);
        return new Font(clone, null, -1, this.createdFont, this.font2DHandle);
    }
    
    public Font deriveFont(int n) {
        if (this.values == null) {
            return new Font(this.name, n, (float)this.size, this.createdFont, this.font2DHandle);
        }
        AttributeValues clone = this.getAttributeValues().clone();
        int n2 = (this.style != n) ? this.style : -1;
        applyStyle(n, clone);
        return new Font(clone, null, n2, this.createdFont, this.font2DHandle);
    }
    
    public Font deriveFont(Map<? extends AttributedCharacterIterator.Attribute, ?> map) {
        if (map == null) {
            return this;
        }
        AttributeValues clone = this.getAttributeValues().clone();
        clone.merge(map, Font.RECOGNIZED_MASK);
        return new Font(clone, this.name, this.style, this.createdFont, this.font2DHandle);
    }
    
    public boolean canDisplay(char c) {
        return this.getFont2D().canDisplay(c);
    }
    
    public boolean canDisplay(int i) {
        if (!Character.isValidCodePoint(i)) {
            throw new IllegalArgumentException("invalid code point: " + Integer.toHexString(i));
        }
        return this.getFont2D().canDisplay(i);
    }
    
    public int canDisplayUpTo(String s) {
        Font2D font2D = this.getFont2D();
        for (int length = s.length(), i = 0; i < length; ++i) {
            int char1 = s.charAt(i);
            if (!font2D.canDisplay((char)char1)) {
                if (!Character.isHighSurrogate((char)char1)) {
                    return i;
                }
                if (!font2D.canDisplay(s.codePointAt(i))) {
                    return i;
                }
                ++i;
            }
        }
        return -1;
    }
    
    public int canDisplayUpTo(char[] a, int i, int limit) {
        Font2D font2D = this.getFont2D();
        int n;
        for (; i < limit; ++i) {
            n = a[i];
            if (!font2D.canDisplay((char)n)) {
                if (!Character.isHighSurrogate((char)n)) {
                    return i;
                }
                if (!font2D.canDisplay(Character.codePointAt(a, i, limit))) {
                    return i;
                }
                ++i;
            }
        }
        return -1;
    }
    
    public int canDisplayUpTo(CharacterIterator characterIterator, int i, int n) {
        Font2D font2D = this.getFont2D();
        int high = characterIterator.setIndex(i);
        int next;
        for (; i < n; ++i, high = characterIterator.next()) {
            if (!font2D.canDisplay((char)high)) {
                if (!Character.isHighSurrogate((char)high)) {
                    return i;
                }
                next = characterIterator.next();
                if (!Character.isLowSurrogate((char)next)) {
                    return i;
                }
                if (!font2D.canDisplay(Character.toCodePoint((char)high, (char)next))) {
                    return i;
                }
                ++i;
            }
        }
        return -1;
    }
    
    public float getItalicAngle() {
        return this.getItalicAngle(null);
    }
    
    private float getItalicAngle(FontRenderContext fontRenderContext) {
        Object aaHint;
        Object fmHint;
        if (fontRenderContext == null) {
            aaHint = RenderingHints.VALUE_TEXT_ANTIALIAS_OFF;
            fmHint = RenderingHints.VALUE_FRACTIONALMETRICS_OFF;
        }
        else {
            aaHint = fontRenderContext.getAntiAliasingHint();
            fmHint = fontRenderContext.getFractionalMetricsHint();
        }
        return this.getFont2D().getItalicAngle(this, Font.identityTx, aaHint, fmHint);
    }
    
    public boolean hasUniformLineMetrics() {
        return false;
    }
    
    private FontLineMetrics defaultLineMetrics(FontRenderContext fontRenderContext) {
        FontLineMetrics referent;
        if (this.flmref == null || (referent = (FontLineMetrics)this.flmref.get()) == null || !referent.frc.equals(fontRenderContext)) {
            float[] metrics = new float[8];
            this.getFont2D().getFontMetrics(this, Font.identityTx, fontRenderContext.getAntiAliasingHint(), fontRenderContext.getFractionalMetricsHint(), metrics);
            float ascent = metrics[0];
            float descent = metrics[1];
            float leading = metrics[2];
            float ssOffset = 0.0f;
            if (this.values != null && this.values.getSuperscript() != 0) {
                ssOffset = (float)this.getTransform().getTranslateY();
                ascent -= ssOffset;
                descent += ssOffset;
            }
            float height = ascent + descent + leading;
            int baselineIndex = 0;
            float[] baselineOffsets = { 0.0f, (descent / 2.0f - ascent) / 2.0f, -ascent };
            float y = metrics[4];
            float y2 = metrics[5];
            float y3 = metrics[6];
            float y4 = metrics[7];
            float italicAngle = this.getItalicAngle(fontRenderContext);
            if (this.isTransformed()) {
                AffineTransform charTransform = this.values.getCharTransform();
                if (charTransform != null) {
                    Point2D.Float o = new Point2D.Float();
                    o.setLocation(0.0f, y);
                    charTransform.deltaTransform(o, o);
                    y = o.y;
                    o.setLocation(0.0f, y2);
                    charTransform.deltaTransform(o, o);
                    y2 = o.y;
                    o.setLocation(0.0f, y3);
                    charTransform.deltaTransform(o, o);
                    y3 = o.y;
                    o.setLocation(0.0f, y4);
                    charTransform.deltaTransform(o, o);
                    y4 = o.y;
                }
            }
            referent = new FontLineMetrics(0, new CoreMetrics(ascent, descent, leading, height, baselineIndex, baselineOffsets, y + ssOffset, y2, y3 + ssOffset, y4, ssOffset, italicAngle), fontRenderContext);
            this.flmref = new SoftReference(referent);
        }
        return (FontLineMetrics)referent.clone();
    }
    
    public LineMetrics getLineMetrics(String s, FontRenderContext fontRenderContext) {
        FontLineMetrics defaultLineMetrics = this.defaultLineMetrics(fontRenderContext);
        defaultLineMetrics.numchars = s.length();
        return defaultLineMetrics;
    }
    
    public LineMetrics getLineMetrics(String s, int n, int n2, FontRenderContext fontRenderContext) {
        FontLineMetrics defaultLineMetrics = this.defaultLineMetrics(fontRenderContext);
        int n3 = n2 - n;
        defaultLineMetrics.numchars = ((n3 < 0) ? 0 : n3);
        return defaultLineMetrics;
    }
    
    public LineMetrics getLineMetrics(char[] array, int n, int n2, FontRenderContext fontRenderContext) {
        FontLineMetrics defaultLineMetrics = this.defaultLineMetrics(fontRenderContext);
        int n3 = n2 - n;
        defaultLineMetrics.numchars = ((n3 < 0) ? 0 : n3);
        return defaultLineMetrics;
    }
    
    public LineMetrics getLineMetrics(CharacterIterator characterIterator, int n, int n2, FontRenderContext fontRenderContext) {
        FontLineMetrics defaultLineMetrics = this.defaultLineMetrics(fontRenderContext);
        int n3 = n2 - n;
        defaultLineMetrics.numchars = ((n3 < 0) ? 0 : n3);
        return defaultLineMetrics;
    }
    
    public Rectangle2D getStringBounds(String s, FontRenderContext fontRenderContext) {
        char[] charArray = s.toCharArray();
        return this.getStringBounds(charArray, 0, charArray.length, fontRenderContext);
    }
    
    public Rectangle2D getStringBounds(String s, int beginIndex, int endIndex, FontRenderContext fontRenderContext) {
        return this.getStringBounds(s.substring(beginIndex, endIndex), fontRenderContext);
    }
    
    public Rectangle2D getStringBounds(char[] text, int n, int n2, FontRenderContext fontRenderContext) {
        if (n < 0) {
            throw new IndexOutOfBoundsException("beginIndex: " + n);
        }
        if (n2 > text.length) {
            throw new IndexOutOfBoundsException("limit: " + n2);
        }
        if (n > n2) {
            throw new IndexOutOfBoundsException("range length: " + (n2 - n));
        }
        int n3 = (this.values == null || (this.values.getKerning() == 0 && this.values.getLigatures() == 0 && this.values.getBaselineTransform() == null)) ? 1 : 0;
        if (n3 != 0) {
            n3 = (FontUtilities.isComplexText(text, n, n2) ? 0 : 1);
        }
        if (n3 != 0) {
            return new StandardGlyphVector(this, text, n, n2 - n, fontRenderContext).getLogicalBounds();
        }
        TextLayout o = new TextLayout(new String(text, n, n2 - n), this, fontRenderContext);
        return new Rectangle2D.Float(0.0f, -o.getAscent(), o.getAdvance(), o.getAscent() + o.getDescent() + o.getLeading());
    }
    
    public Rectangle2D getStringBounds(CharacterIterator characterIterator, int n, int i, FontRenderContext fontRenderContext) {
        int beginIndex = characterIterator.getBeginIndex();
        int endIndex = characterIterator.getEndIndex();
        if (n < beginIndex) {
            throw new IndexOutOfBoundsException("beginIndex: " + n);
        }
        if (i > endIndex) {
            throw new IndexOutOfBoundsException("limit: " + i);
        }
        if (n > i) {
            throw new IndexOutOfBoundsException("range length: " + (i - n));
        }
        char[] array = new char[i - n];
        characterIterator.setIndex(n);
        for (int j = 0; j < array.length; ++j) {
            array[j] = characterIterator.current();
            characterIterator.next();
        }
        return this.getStringBounds(array, 0, array.length, fontRenderContext);
    }
    
    public Rectangle2D getMaxCharBounds(FontRenderContext frc) {
        float[] metrics = new float[4];
        this.getFont2D().getFontMetrics(this, frc, metrics);
        return new Rectangle2D.Float(0.0f, -metrics[0], metrics[3], metrics[0] + metrics[1] + metrics[2]);
    }
    
    public GlyphVector createGlyphVector(FontRenderContext frc, String str) {
        return new StandardGlyphVector(this, str, frc);
    }
    
    public GlyphVector createGlyphVector(FontRenderContext frc, char[] text) {
        return new StandardGlyphVector(this, text, frc);
    }
    
    public GlyphVector createGlyphVector(FontRenderContext frc, CharacterIterator iter) {
        return new StandardGlyphVector(this, iter, frc);
    }
    
    public GlyphVector createGlyphVector(FontRenderContext frc, int[] glyphs) {
        return new StandardGlyphVector(this, glyphs, frc);
    }
    
    public GlyphVector layoutGlyphVector(FontRenderContext frc, char[] text, int offset, int n, int flags) {
        GlyphLayout value = GlyphLayout.get(null);
        StandardGlyphVector layout = value.layout(this, frc, text, offset, n - offset, flags, null);
        GlyphLayout.done(value);
        return layout;
    }
    
    private static void applyTransform(AffineTransform transform, AttributeValues attributeValues) {
        if (transform == null) {
            throw new IllegalArgumentException("transform must not be null");
        }
        attributeValues.setTransform(transform);
        return;
    }
    
    private static void applyStyle(int n, AttributeValues attributeValues) {
        attributeValues.setWeight(((n & 0x1) != 0x0) ? 2.0f : 1.0f);
        attributeValues.setPosture(((n & 0x2) != 0x0) ? 0.2f : 0.0f);
    }
    
    private static native void initIDs();
    
    static Font2D access$000(Font font) {
        return font.getFont2D();
    }
    
    static {
        Toolkit.loadLibraries();
        initIDs();
        FontAccess.setFontAccess(new Font.FontAccessImpl());
        identityTx = new AffineTransform();
        RECOGNIZED_MASK = (AttributeValues.MASK_ALL & ~AttributeValues.getMask(EAttribute.EFONT));
        PRIMARY_MASK = AttributeValues.getMask(new EAttribute[] { EAttribute.EFAMILY, EAttribute.EWEIGHT, EAttribute.EWIDTH, EAttribute.EPOSTURE, EAttribute.ESIZE, EAttribute.ETRANSFORM, EAttribute.ESUPERSCRIPT, EAttribute.ETRACKING });
        SECONDARY_MASK = (Font.RECOGNIZED_MASK & ~Font.PRIMARY_MASK);
        LAYOUT_MASK = AttributeValues.getMask(new EAttribute[] { EAttribute.ECHAR_REPLACEMENT, EAttribute.EFOREGROUND, EAttribute.EBACKGROUND, EAttribute.EUNDERLINE, EAttribute.ESTRIKETHROUGH, EAttribute.ERUN_DIRECTION, EAttribute.EBIDI_EMBEDDING, EAttribute.EJUSTIFICATION, EAttribute.EINPUT_METHOD_HIGHLIGHT, EAttribute.EINPUT_METHOD_UNDERLINE, EAttribute.ESWAP_COLORS, EAttribute.ENUMERIC_SHAPING, EAttribute.EKERNING, EAttribute.ELIGATURES, EAttribute.ETRACKING, EAttribute.ESUPERSCRIPT });
        EXTRA_MASK = AttributeValues.getMask(new EAttribute[] { EAttribute.ETRANSFORM, EAttribute.ESUPERSCRIPT, EAttribute.EWIDTH });
        ssinfo = new float[] { 0.0f, 0.375f, 0.625f, 0.7916667f, 0.9027778f, 0.9768519f, 1.0262346f, 1.0591564f };
    }
    
    private static class FontAccessImpl extends FontAccess
    {
        @Override
        public Font2D getFont2D(Font font) {
            return Font.access$000(font);
        }
        
        @Override
        public void setFont2D(Font font, Font2DHandle font2DHandle) {
            font.font2DHandle = font2DHandle;
        }
        
        @Override
        public void setCreatedFont(Font font) {
            font.createdFont = true;
        }
        
        @Override
        public boolean isCreatedFont(Font font) {
            return font.createdFont;
        }
    }
}
